package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.streams.StreamObject;

import java.io.File;
import java.io.IOException;
import java.util.List;

public class AudioConverter extends Converter {
    private String newExt;

    /**
     * Instantiates a new audio converter
     * @param ffprobePath <p>Path/command to ffprobe.</p>
     * @param ffmpegPath <p>Path/command to ffmpeg.</p>
     * @param newExt <p>The extension of the new file.</p>
     */
    public AudioConverter(String ffprobePath, String ffmpegPath, String newExt) {
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
        this.newExt = newExt;
    }

    /**
     * Processes a file conversion
     * @param folder <p>The work folder containing the file.</p>
     * @param file <p>The file to convert.</p>
     * @param newExt <p>The extension of the new file.</p>
     * @throws IOException <p>If the file cannot be converted.</p>
     */
    private void processFile(File folder, File file, String newExt) throws IOException {
        List<StreamObject> streams = probeFile(ffprobePath, file);
        if (streams.size() == 0) {
            throw new IllegalArgumentException("The file has no streams");
        }
        String newPath = stripExtension(file) + "." + newExt;
        convertProcess(new ProcessBuilder(builderCommand(ffmpegPath, file.getName(), streams, newPath)), folder);
    }

    /**
     * Generates a command for a ProcessBuilder.
     * @param executable <p>The executable file for ffmpeg.</p>
     * @param fileName <p>The input file.</p>
     * @param streams <p>A list of ffprobe streams.</p>
     * @param outFile <p>The output file.</p>
     * @return <p>A list of commands.</p>
     */
    private String[] builderCommand(String executable, String fileName, List<StreamObject> streams, String outFile) {
        List<String> command = generalFile(executable, fileName);
        List<AudioStream> audioStreams = filterStreamsByType(streams, "audio");
        AudioStream audioStream = null;
        if (audioStreams.size() > 0) {
            audioStream = audioStreams.get(0);
        }
        if (audioStreams.size() > 0) {
            command.add("-map");
            command.add("0:" + audioStream.getAbsoluteIndex());
        }
        command.add(outFile);
        return command.toArray(new String[0]);
    }

    @Override
    public String[] getValidFormats() {
        return AUDIO_FORMATS;
    }

    @Override
    public void convert(File file) throws IOException {
        processFile(file.getParentFile(), file, newExt);
    }
}